/*
===============================================================================
===============================================================================

file:   I2C.c
by:     Lukasz Krzak & Marcin Krzesinski
date:   16.07.2004

description:   File contains I2C service functions


===============================================================================
===============================================================================
*/

/* The following definition causes compilation of a uC/OS-II compatible version.
   For standard applications this should be removed. */

#define UCOS_PROJECT

/* For uC/OS version only file "includes.h" is included. For non-uC/OS projects
   a standard set of header files is included. */
#ifdef UCOS_PROJECT
#warning "I2C: Compiling uC/OS-II version"
#include <includes.h>
#else
#include<avr/stdio.h>
#include<avr/stdlib.h>
#include<avr/io.h>
#include<avr/pgmspace.h>
#include<Buffers/Buffers.h>
#include<I2C.h>
#endif

#define I2C_ISR()   SIGNAL(SIG_2WIRE_SERIAL)


I2C_ISR()
{

  #ifdef UCOS_PROJECT
  OS_ENTER_CRITICAL();
  #endif
  if (PORTC & 0x02) cbi(PORTC,PC1);
  else sbi (PORTC,PC1);
  
  switch (TWSR)
  {
    case 0x60:
    case 0x70:
    case 0x80:
    case 0x90:
                    WriteBuffer(I2CBuffer,TWDR);
                    if (PORTC & 0x04) cbi(PORTC,PC2);
                    else sbi (PORTC,PC2);
                    TWCR |= 0x40;        // set ACK operation   TWEA = 1
                    TWCR |= 0x80;        // clear intrrupt flag  TWINT = 1
                    break;
    default :
                    TWCR |= 0x80;        // clear intrrupt flag  TWINT = 1
                    break;
  }
  #ifdef UCOS_PROJECT
  OS_EXIT_CRITICAL();
  #endif
}

int8_t I2C_Init(int8_t Mode)
{
  if (Mode == I2C_SLAVE_RECEIVER)
  {
    I2CBuffer = CreateBuffer(32); // create receive buffer
    TWAR = 0xf1;  // slave address = 1111000 + respond to general calls (LSB=1)
    TWCR = 0x45;  // set slave receiver mode and enable TWI interrupt
  }
  return 0;
} /* I2C_Init */



