/*
===============================================================================
===============================================================================

file:   USART.h
by:     Lukasz Krzak & Marcin Krzesinski
date:   29.06.2004

description:   File contains USART service function headers


===============================================================================
===============================================================================
*/

#define USART_NoOptions                  0x00

/* USARTNo: */
#define USART0                           0x00
#define USART1                           0x01

/* USARTMode: */
#define USART_Asynchronous               0x00
#define USART_Synchronous                0x40

/* PSOptions: */
#define USART_ParityDisabled             0x00
#define USART_ParityEven                 0x20
#define USART_ParityOdd                  0x30
#define USART_DoubleStopBit              0x08

/* IE Options: */
#define USART_RXDIntEnable               0x80
#define USART_TXDIntEnable               0x40
#define USART_DataRegEmptyIntEnable      0x40

/* OtherOptions */
#define USART_RXDBuffer                  0x08
#define USART_TXDBuffer                  0x04
#define USART_DoubleSpeed                0x02
#define USART_MultiProcessorMode         0x01

/* system logs */
#define LOG_USART                        USART0   // USART used for system logs

/* size of USART buffers */
#define USART0_RXD_BUFFER_SIZE           10
#define USART1_RXD_BUFFER_SIZE           64
#define USART0_TXD_BUFFER_SIZE           64
#define USART1_TXD_BUFFER_SIZE           64

/* types of memory the data can be stored in: */
#define MEM_RAM                          0
#define MEM_EEPROM                       1    // not implemented yet
#define MEM_FLASH                        2


/* USART Error Codes: */
#define USART_OK                           0
#define USART_NOT_INITIALIZED              1
#define USART_OUT_OF_MEMORY                10
#define USART_BUFFER_NOT_INITIALIZED       11
#define USART_BUFFER_OVERRUN               12
#define USART_BUFFER_EMPTY                 13
#define USART_BUFFERING_DISABLED           14

#define USART0_MUTEXPRIO 5
#define USART1_MUTEXPRIO 6

#ifdef UCOS_PROJECT
OS_EVENT *USART0Mutex;
OS_EVENT *USART1Mutex;
#endif

/* function headers: */
void USART_Init(int8_t  USARTNo,           /* USARTu number                         */
                int8_t  USARTMode,         /* Async/Sync                            */
                int16_t BaudRatePresc,     /* clock prescaler                       */
                int8_t  CharSize,          /* bits per char (5-9)                   */
                int8_t  PSOptions,         /* parity i stop bit options             */
                int8_t  IEOptions,         /* interrupt enable options              */
                int8_t  OtherOptions       /* additional opcje                      */
                );

int8_t USART_GetChar(int8_t  USARTNo         /* USART number  */
                    );

void USART_PutChar(int8_t USARTNo,           /* USART number      */
                     char Character          /* character to send */
                     );

void USART_PutStr(int8_t USARTNo,           /* USART number            */
                     const char String[],   /* characters to send      */
                     int8_t Location        /* type of memory          */
                     );
                     
int8_t USART_DataReceived(int8_t USARTNo    /* USARTu number    */
                          );
                          
void SystemLog(const char *String  // system log text
              );

void USART_RXD_ISR(void);

int8_t USART_CreateRXDBuffer(int8_t USARTNo,    /* USARTu number    */
                           int8_t BufferSize    /* Size of receive-buffer */
                          );
                          
int8_t USART_WriteRXDBuffer (int8_t USARTNo,
                            char Byte);
                            
int8_t USART_ReadRXDBuffer (int8_t USARTNo,
                            char *Destination,
                            int8_t Count);

int8_t USART_RXDBufferCount(int8_t USARTNo);
