/*
===============================================================================
===============================================================================

file:   VT100.c
by:     Marcin Krzesinski & Lukasz Krzak
date:   08.07.2004

description:   File contains VT100 interface procedures


===============================================================================
===============================================================================
*/

/* Ponizsza dyrektywa powoduje kompilacje pliku w wersji przystosowanej do
   wspolpracy z systemem uC/OS-II. Dla "zwyklych" projektow powinna zostac
   usunieta.                                                                 */
#define UCOS_PROJECT


#ifdef UCOS_PROJECT
#warning "VT100: Compiling uC/OS-II version"
#include <includes.h>
#else
#include<avr/stdio.h>
#include<avr/stdlib.h>
#include<USART.h>
#endif

char CSI[3] = {'\033','[','\0'}; // Control Sequence Introducer




void VT100_CursorPosition (int8_t x,int8_t y) //moves the active position to
{                                             //the position specified by the parameters
   char x_str[4], y_str[4];
   
   itoa (x, x_str, 10);
   itoa (y, y_str, 10);


   USART_PutStr (USART0, CSI, MEM_RAM);
   USART_PutStr (USART0, x_str, MEM_RAM);
   USART_PutChar (USART0, ';');
   USART_PutStr (USART0, y_str, MEM_RAM);
   USART_PutChar (USART0, 'H');

}


void VT100_SaveCursor (void)
{
  USART_PutStr (USART0, "\033\067", MEM_RAM);

}

void VT100_RestoreCursor (void)
{
  USART_PutStr (USART0, "\033\070", MEM_RAM);

}


void VT100_CursorForward (int8_t count)  //moves the active position to the right
{
   char count_str[4];

  itoa (count, count_str, 10);
  USART_PutStr (USART0, CSI, MEM_RAM);
  USART_PutStr (USART0, count_str, MEM_RAM);
  USART_PutChar (USART0, 'C');

}

void VT100_CursorBackword (int8_t count)   // moves the active position to the left
{                                          //  0,1 <=> backspace

  char count_str[4];
  
  itoa (count, count_str, 10);
  USART_PutStr (USART0, CSI, MEM_RAM);
  USART_PutStr (USART0, count_str, MEM_RAM);
  USART_PutChar (USART0, 'D');

}


void VT100_CursorDown (int8_t count)  //moves the active position downward
{                                     // without altering the column position
                                      //0,1 -active position is moved one line downward
   char count_str[4];

  itoa (count, count_str, 10);
  USART_PutStr (USART0, CSI, MEM_RAM);
  USART_PutStr (USART0, count_str, MEM_RAM);
  USART_PutChar (USART0, 'B');

}


                                      //Moves the active position upward without
void VT100_CursorUp (int8_t count)  //altering the column position
{
   char count_str[4];

  itoa (count, count_str, 10);
  USART_PutStr (USART0, CSI, MEM_RAM);
  USART_PutStr (USART0, count_str, MEM_RAM);
  USART_PutChar (USART0, 'A');

}


void VT100_SetMode (int8_t mode)
{
  USART_PutStr (USART0, CSI, MEM_RAM);
  USART_PutChar (USART0, mode+48);
  USART_PutChar (USART0, 'h');
}

void VT100_ResetMode (int8_t mode)
{
  USART_PutStr (USART0, CSI, MEM_RAM);
  USART_PutChar (USART0, mode+48);
  USART_PutChar (USART0, 'l');
}


void VT100_EraseInDisplay (int8_t mode)
{
  USART_PutStr (USART0, CSI, MEM_RAM);
  USART_PutChar (USART0, mode+48);
  USART_PutChar (USART0, 'J');
}



void VT100_EraseInLine (int8_t mode)
{
  USART_PutStr (USART0, CSI, MEM_RAM);
  USART_PutChar (USART0, mode+48);
  USART_PutChar (USART0, 'K');
}


void VT100_SetTopAndBottomMarg (int8_t top,int8_t bottom)
{
   char top_str[4], bottom_str[4];

   itoa (top, top_str, 10);
   itoa (bottom, bottom_str, 10);


   USART_PutStr (USART0, CSI, MEM_RAM);
   USART_PutStr (USART0, top_str, MEM_RAM);
   USART_PutChar (USART0, ';');
   USART_PutStr (USART0, bottom_str, MEM_RAM);
   USART_PutChar (USART0, 'r');

}

void VT102_TextAttribute(int8_t attribute)
{
    USART_PutStr (USART0, CSI, MEM_RAM);
    switch (attribute)
    {
      case SET_BOLD:
                      USART_PutChar (USART0, '1');
                      break;
      case SET_UNDERSCORE:
                      USART_PutChar (USART0, '4');
                      break;
      case SET_BLINK:
                      USART_PutChar (USART0, '5');
                      break;
      case SET_NEGATIVE:
                      USART_PutChar (USART0, '7');
                      break;
      case RESET_BOLD:
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, '2');
                      break;
      case RESET_UNDERSCORE:
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, '4');
                      break;
      case RESET_BLINK:
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, '5');
                      break;
      case RESET_NEGATIVE:
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, '7');
                      break;
      case SET_NORMAL:
      default:
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, ';');
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, '4');
                      USART_PutChar (USART0, ';');
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, '5');
                      USART_PutChar (USART0, ';');
                      USART_PutChar (USART0, '2');
                      USART_PutChar (USART0, '7');
                      break;
    }
    USART_PutChar (USART0, attribute );
    USART_PutChar (USART0, 'm');
}




