/*
===============================================================================
===============================================================================

file:   VT100.h
by:     Marcin Krzesinski & Lukasz Krzak
date:   08.07.2004

description:   File contains VT100 interface function headers


===============================================================================
===============================================================================
*/




/*   SetMode / ResetMode modes:                                                      */
#define CURSORKEY  1 // for keypad aplications only
#define ANSI_VT52  2 // only ANSI(S)/VT52(R) compatible escape sequences are interpreted and executed
#define COLUMN     3 // maximum of 132(S)/80(R) columns on the screen
#define SCROLLING  4 // (S) - smooth scroll, (R) - otherwise
#define SCREEN     5 // (S) - white with black characters, (R) - otherwise,
#define ORIGIN     6 // Line and column numbers are(S)/aren't(R) independent of current margin settings
#define AUTOWRAP   7 // wrap text when (S)
#define AUTOREPEAT 8 // (S) causes keyboard keys to auto-repeat
#define INTERLACE  9 // when (S) causes the video processor to display 480 scan lines per frame

/*   EraseInDisplay / EraseInLine modes:                                                      */
#define TOTHEEND   0
#define FROMSTART  1
#define ALL        2

/* TextAttributes:                                                */
#define SET_NORMAL       0
#define SET_BOLD         1
#define SET_UNDERSCORE   2
#define SET_BLINK        3
#define SET_NEGATIVE     4
#define RESET_BOLD       5
#define RESET_UNDERSCORE 6
#define RESET_BLINK      7
#define RESET_NEGATIVE   8

void VT100_CursorPosition (int8_t x,int8_t y);//moves the active position to
                                             //the position specified by the parameters
void VT100_SaveCursor (void);

void VT100_RestoreCursor (void);

void VT100_CursorForward (int8_t count);  //moves the active position to the right

void VT100_CursorBackword (int8_t count);  // moves the active position to the left
                                           //  0,1 <=> backspace
                                           
void VT100_CursorDown (int8_t count);  //moves the active position downward
                                       // without altering the column position
                                       //0,1 -active position is moved one line downward
                                       
void VT100_CursorUp (int8_t count); //moves the active position upward
                                    // without altering the column position
                                    //0,1 -active position is moved one line upward
                                    
void VT100_SetMode (int8_t mode);

void VT100_ResetMode (int8_t mode);

void VT100_EraseInDisplay (int8_t mode);

void VT100_EraseInLine (int8_t mode);

void VT100_SetTopAndBottomMarg (int8_t top,int8_t bottom);

void VT102_TextAttribute(int8_t attribute);
