/*
===============================================================================
===============================================================================

plik:   tasks.c
autor:  Lukasz Krzak & Marcin Krzesinski
data:   4.07.2004

opis:   Plik zawierajacy implementacje zada (wtkw) uruchamianych
        w systemie uC/OS-II na procesorze AVR


===============================================================================
===============================================================================
*/

/* INCLUDES */
#include<includes.h>


/* LOGi watku IOTask */
static prog_char ResetAVRCommand[]="Reseting AVR..\n\r";
static prog_char ResetUSBCommand[]="Reseting USB Controller..\n\r";
static prog_char StatisticsCommand[]="Statistics : ";
static prog_char SystemLogsCommand[]="System Logs : ";
static prog_char I2CMonitorCommand[]="I2C Monitoring : ";
static prog_char On[]="On \n\r";
static prog_char Off[]="Off\n\r";
static prog_char LogSystemReady[]="System ready!\n\r";
static prog_char Header[]=
  "uC/OS-II running on Atmel AVR ATMega128                                      \n\r";
static prog_char SystemManual[8][80] =
{

  "+------------------------------------------+---[CPU:     ]-[TIME:         ]-+\n\r",
  "|              RS232 COMMANDS              |     CURRENT SETTINGS           |\n\r",
  "| 'r' - AVR reset                          |                                |\n\r",
  "| 'c' - CY7c68013 reset                    |                                |\n\r",
  "| 's' - CPU usage statistics on/off        |                                |\n\r",
  "| 'l' - system logs on/off                 |                                |\n\r",
  "| 'i' - I2C monitoring on/off              |                                |\n\r",
  "+------------------------------------------+--------------------------------+\n\r"
};

#define DELAYCOUNT 50;
int8_t DelayCount;

#define I2C_IDLE                     0x00
#define I2C_LCDWRITE                 0x80

#define I2C_COM_WRITE_LCD_START       0xf0
#define I2C_COM_WRITE_LCD_STOP        0xf1

uint8_t I2C_State;        // Holds current I2C interface state (for state machine in IOTask)
uint8_t I2C_CommCount;    // Holds current command data count
char   I2C_CommBuf[12];  // I2C command buffer (used to hold command data)
/*
-------------------------------------------------------------------------------
IOTask
~~~~~~

Wtek odpowiedzialny za komunikacj z urzdzeniami zewntrznymi.
Aktualnie odopowiada za obslug: USART(RS232), LCD, I2C

POLECENIA KONSOLI RS232:
    'r' - resetuje procesor
    'c' - resetuje doczony ukad CY7c68013
    's' - wlacza/wylacza monitorowanie zuzycia procesora
    'l' - wlacza/wylacza wysylanie logw systemu
    'i' - wlacza/wylacza monitorowanie magistrali I2C

Parametry:

  pdata [i/o]  - aktualnie nieuywany

-------------------------------------------------------------------------------
*/
void IOTask (void *pdata)
{
  char znak;
  char tmp[4];
  
  for (;;)
  {
     if (DelayCount > 0 )
         DelayCount--;
     if ((DelayCount) == 1)
     {
         OSMutexPend(LCDMutex,0,NULL); // gain access to LCD resource
         LCD_Goto(0,0);
         if (VAR_ShowCPUUsage != 0)
            LCD_WriteText("CPU Usage:      ");
         else
            LCD_WriteText("Ready !         ");
         OSMutexPost(LCDMutex);       // return access to LCD resource
     }

     /* Process I2C Buffer: */
     if (VAR_I2CMonitor != 0)
     while (BufferCount(I2CBuffer) != 0)
     {
          ReadBuffer(I2CBuffer,&znak,1);  // read I2C receive buffer
          
          /* Update I2C interface state: */
          switch (znak)
          {
            case I2C_COM_WRITE_LCD_START:   // enter LCD display mode
                                           I2C_State = I2C_LCDWRITE;
                                           I2C_CommCount = 0;
                                           cbi(PORTC,PC4);
                                           break;
                                           
            case I2C_COM_WRITE_LCD_STOP:   // display received text on LCD:
                                           //I2C_CommBuf[I2C_CommCount] = 0; // add null-termination
                                           OSMutexPend(LCDMutex,0,NULL); // gain access to LCD resource
                                           LCD_ClrLine(0);
                                           LCD_Goto(0,0);
                                           LCD_WriteText("I2C: ");
                                           LCD_WriteText(I2C_CommBuf);
                                           OSMutexPost(LCDMutex);       // return access to LCD resource
                                           itoa(I2C_CommCount,tmp,10);
                                           OSMutexPend(USART0Mutex,0,NULL); //  gain access to USART0 resource
                                           USART_PutStr(USART0, "I2C[", MEM_RAM);
                                           USART_PutStr(USART0, tmp, MEM_RAM);
                                           USART_PutStr(USART0, "]: ", MEM_RAM);
                                           USART_PutStr(USART0, I2C_CommBuf, MEM_RAM);
                                           USART_PutStr(USART0, "<-\n\r", MEM_RAM);
                                           OSMutexPost(USART0Mutex); //return access to USART0 resource

                                           DelayCount = DELAYCOUNT;
                                           sbi(PORTC,PC4);
                                           // exit LCD display mode and return to idle
                                           I2C_State = I2C_IDLE;
                                           break;
            default:
                   /* Execute I2C_State-dependent procedures: */
                    if (I2C_State == I2C_LCDWRITE)
                    {
                                           I2C_CommBuf[I2C_CommCount] = znak;
                                           if (I2C_CommCount < 11)
                                              I2C_CommCount++;
                    }
                                           break;
            } // switch (znak)
          

     } // while (BufferCount(I2CBuffer) != 0)
     /* Process USART0 Buffer: */
     while (USART_RXDBufferCount(USART0) != 0)
     {
       USART_ReadRXDBuffer(USART0,&znak,1);
       switch (znak)
       {
         case 'r':
         case 'R':  // AVR reset:
                    OSMutexPend(LCDMutex,0,NULL); // gain access to LCD resource
                    LCD_Goto(0,0);
                    LCD_WriteText("AVR Reset..     ");
                    OSMutexPost(LCDMutex);       // return access to LCD resource

                    OSMutexPend(USART0Mutex,0,NULL); //  gain access to USART0 resource
                    SystemLog(ResetAVRCommand);
                    OSMutexPost(USART0Mutex); //return access to USART0 resource

                    _delay_loop_2(32000);
                    asm("jmp 0x0000");
                    DelayCount = DELAYCOUNT;
                    break;

         case 'c':
         case 'C':  // USB CY7C68013 reset:
                    OSMutexPend(LCDMutex,0,NULL); // gain access to LCD resource
                    LCD_Goto(0,0);
                    LCD_WriteText("USB Reset..     ");
                    OSMutexPost(LCDMutex);       // return access to LCD resource
                    
                    OSMutexPend(USART0Mutex,0,NULL); //  gain access to USART0 resource
                    SystemLog(ResetUSBCommand);
                    OSMutexPost(USART0Mutex); //return access to USART0 resource

                    DDRB |= 0x40;
                    cbi(PORTB,PB6);
                    _delay_loop_2(32000);
                    sbi(PORTB,PB6);
                    DelayCount = DELAYCOUNT;
                    break;
         case 's':
         case 'S':  // statistics on/off
                    if (VAR_ShowCPUUsage != 0)
                      VAR_ShowCPUUsage = 0;
                    else
                      VAR_ShowCPUUsage = 1;

                    OSMutexPend(LCDMutex,0,NULL); // gain access to LCD resource
                    LCD_Goto(0,0);
                    if (VAR_ShowCPUUsage == 0)
                    {
                       LCD_WriteText("Statistics: OFF ");
                       LCD_ClrLine(1);
                    }
                    else
                       LCD_WriteText("Statistics: ON ");
                    OSMutexPost(LCDMutex);       // return access to LCD resource

                    OSMutexPend(USART0Mutex,0,NULL); //  gain access to USART0 resource
                    SystemLog(StatisticsCommand);
                    if (VAR_ShowCPUUsage == 0)
                       SystemLog(Off);
                    else
                       SystemLog(On);
                    VT100_SaveCursor();
                    VT100_CursorPosition (4,61);

                    if (VAR_ShowCPUUsage == 0)
                      USART_PutStr(USART0, "Off", MEM_RAM);
                    else
                      USART_PutStr(USART0, "On ", MEM_RAM);
                    VT100_RestoreCursor();
                    OSMutexPost(USART0Mutex); //return access to USART0 resource
                      //
                    break;
         case 'l':
         case 'L':  // system logs on/off
                    if (VAR_SystemLogs != 0)
                      VAR_SystemLogs = 0;
                    else
                      VAR_SystemLogs = 1;

                    OSMutexPend(LCDMutex,0,NULL); // gain access to LCD resource
                    LCD_Goto(0,0);
                    if (VAR_SystemLogs == 0)
                    {
                       LCD_WriteText("System logs: OFF");
                       LCD_ClrLine(1);
                    }
                    else
                       LCD_WriteText("System logs: ON ");
                    OSMutexPost(LCDMutex);       // return access to LCD resource

                    OSMutexPend(USART0Mutex,0,NULL); //  gain access to USART0 resource
                    SystemLog(SystemLogsCommand);
                    if (VAR_SystemLogs == 0)
                       SystemLog(Off);
                    else
                       SystemLog(On);
                    VT100_SaveCursor();
                    VT100_CursorPosition (5,61);

                    if (VAR_SystemLogs == 0)
                      USART_PutStr(USART0, "Off", MEM_RAM);
                    else
                      USART_PutStr(USART0, "On ", MEM_RAM);
                    VT100_RestoreCursor();
                    OSMutexPost(USART0Mutex); //return access to USART0 resource
                      //
                    break;

         case 'i':
         case 'I':  // I2C monitoring on/off
                    if (VAR_I2CMonitor != 0)
                      VAR_I2CMonitor = 0;
                    else
                      VAR_I2CMonitor = 1;

                    OSMutexPend(LCDMutex,0,NULL); // gain access to LCD resource
                    LCD_Goto(0,0);
                    if (VAR_I2CMonitor == 0)
                    {
                       LCD_WriteText("I2CMonitor: OFF");
                       LCD_ClrLine(1);
                    }
                    else
                       LCD_WriteText("I2CMonitor: ON ");
                    OSMutexPost(LCDMutex);       // return access to LCD resource

                    OSMutexPend(USART0Mutex,0,NULL); //  gain access to USART0 resource
                    SystemLog(I2CMonitorCommand);
                    if (VAR_I2CMonitor == 0)
                       SystemLog(Off);
                    else
                       SystemLog(On);
                    VT100_SaveCursor();
                    VT100_CursorPosition (6,65);

                    if (VAR_I2CMonitor == 0)
                      USART_PutStr(USART0, "Off", MEM_RAM);
                    else
                      USART_PutStr(USART0, "On ", MEM_RAM);
                    VT100_RestoreCursor();
                    OSMutexPost(USART0Mutex); //return access to USART0 resource
                      //
                    break;
         default:
                    OSMutexPend(LCDMutex,0,NULL); // gain access to LCD resource
                    LCD_Goto(0,0);
                    LCD_WriteText("USART [R]:     ");
                    LCD_Goto(11,0);
                    LCD_WriteByte(znak,1);
                    OSMutexPost(LCDMutex);       // return access to LCD resource
                    DelayCount = DELAYCOUNT;
                    break;
       }

     }
     OSTimeDly(5);
  }

} /* IOTask */


/*
-------------------------------------------------------------------------------
LEDTask
~~~~~~~

Wtek steruje miganiem diody dolaczonej do portu C (pin PC3)

Parametry:

  pdata [i/o]  - aktualnie nieuywany

-------------------------------------------------------------------------------
*/
void LEDTask(void *pdata)
{
  for (;;)
  {
    OS_ENTER_CRITICAL();
    sbi(PORTC,PC3);
    OS_EXIT_CRITICAL();

    OSTimeDly(10);

    OS_ENTER_CRITICAL();
    cbi(PORTC,PC3);
    OS_EXIT_CRITICAL();

    OSTimeDly(10);
  }
} /* LEDTask */

/*
-------------------------------------------------------------------------------
CalcTask
~~~~~~~

Wtek dokonuje prostych obliczen - symuluje obciaenie procesora

Parametry:

  pdata [i/o]  - aktualnie nieuywany

-------------------------------------------------------------------------------
*/
void CalcTask(void *pdata)
{
  int16_t i;
  int16_t a,b,c;
  for (;;)
  {
    for (i=1;i<32000;i++)
     {
      a = i * 2;
      b = a - 2;
     }

     OSTimeDly(9);




     for (i=1;i<32000;i++)
     {
      a = i * 2;
      b = (a - 2) * a;
      c = b^15;

     }

     OSTimeDly(2);
  }
} /* CalcTask */

/*
-------------------------------------------------------------------------------
StartTask
~~~~~~~~~

Ten wtek uruchamia wszystkie inne zadania w systemie.
Uruchamianie zada z osobnego wtku (a nie bezposrednia z main() ) zostao
zastosowane w celu umoliwienia prawidlowej pracy wtku odpowiedzialnego za
statystyk zuycia procesora. Aby statystyka byla wiarygodna system musi przez
pierwsze 2 sekundy dzialac w stanie "jaowym" tzn. bez dodatkowych zada, aby
wtek statystyczny mogl obliczy wydajno obliczeniow nieobcionego systemu.
Na tej podstawie liczone jest poniej procentowe zuycie procesora.

Parametry:

  pdata [i/o]  - aktualnie nieuywany

-------------------------------------------------------------------------------
*/
void StartTask (void *pdata)
{
  int8_t i;
  OSStatInit();  // Statistics task initialization

  DelayCount = DELAYCOUNT;
  I2C_State = I2C_IDLE;
  I2C_CommCount = 0;

  /* Create mutual exclusion semaphores for all shared resources: */
  USART0Mutex = OSMutexCreate(USART0_MUTEXPRIO, NULL);
  //USART1Mutex = OSMutexCreate(USART1_MUTEXPRIO, NULL);
  LCDMutex = OSMutexCreate(LCD_MUTEXPRIO, NULL);
  
  /* Create all tasks: */
  OSTaskCreate(LEDTask, (void *)0, (void *)&LEDTaskStk[TASK_STK_SIZE - 1], 8);
  OSTaskCreate(CalcTask, (void *)0, (void *)&CalcTaskStk[TASK_STK_SIZE - 1], 9);
  OSTaskCreate(IOTask, (void *)0, (void *)&IOTaskStk[TASK_STK_SIZE - 1], 7);

  OSMutexPend(USART0Mutex,0,NULL); //  gain access to USART0 resource

  SystemLog(LogSystemReady);
  
  VT100_CursorPosition(0,0);
  VT102_TextAttribute(SET_NEGATIVE);
  SystemLog(Header);
  VT102_TextAttribute(SET_NORMAL);

  /* display system manual: */
  for (i=0; i<8; i++)
    SystemLog(SystemManual[i]);
  
  /* display current settings: */
  
    /* CPU statistics: */
    VT100_CursorPosition (4,48);
    SystemLog(StatisticsCommand);
    SystemLog(On);

    /* System LOGs: */
    VT100_CursorPosition (5,48);
    SystemLog(SystemLogsCommand);
    SystemLog(On);
  
    /* I2C monitoring: */
    VT100_CursorPosition (6,48);
    SystemLog(I2CMonitorCommand);
    SystemLog(On);

  /* Set the scrolling area of display: */
  VT100_SetTopAndBottomMarg (11, 21);
  VT100_CursorPosition (11,0);
  
  OSMutexPost(USART0Mutex);    //return access to USART0 resource

  /* At this point all initialization code is executed. Now we can enable
     interrupts to allow context switching. */

  OS_EXIT_CRITICAL();    // enable interrupts (disabled in main function)

  /* This task is now obsolete and can be suspended (or deleted) */
  OSTaskSuspend(OS_PRIO_SELF);  //
  
} /* StartTask */

