/*
===============================================================================
===============================================================================

file:   test.c
by:     Lukasz Krzak & Marcin Krzesinski
date:   22.06.2004

description:   Project main file (TARGET)


===============================================================================
===============================================================================
*/


/* INCLUDES  */
#include "includes.h"    //  main header file grouping all headers



/* Variables allocated in program memory: */
/* Some system LOGS : */
static prog_char InitLog1[]="uC/OS-II running on Atmel AVR ATMega128\n\r";
static prog_char InitLog2[]="Initializing..\n\r";
static prog_char InitLog3[]="   Configuring ports..\n\r";
static prog_char InitLog4[]="   Setting timer ISR..\n\r";
static prog_char InitLog5[]="   Initializing LCD..\n\r";
static prog_char InitLog6[]="   Initializing system..\n\r";


/*
-------------------------------------------------------------------------------
AvrInit
~~~~~~~

This procedure initializes AVR processor to work with uC/OS-II system

Parameters: none

-------------------------------------------------------------------------------
*/
void AvrInit (void)
{

    outp(0x07,	TCCR0);   /* Set TIMER0 prescaler to CLK/1024                 */
    outp(0x01,  TIMSK);   /* Enable TIMER0 overflow interrupt                 */
} /* AvrInit */


/*
-------------------------------------------------------------------------------
main
~~~~

This is the main function for the target project.

Parameters: none

-------------------------------------------------------------------------------
*/
int main (void)
{
  OS_ENTER_CRITICAL();  // disable all interrupts (reenabled in StartTask)
  
  /* Initialize I2C service */
  I2C_Init(I2C_SLAVE_RECEIVER);

  /* Set initial value of global variables */
  VAR_ShowCPUUsage = 1; // show CPU statistics
  VAR_SystemLogs = 1;   // send system logs (through USART0)
  VAR_I2CMonitor = 1;   // enable I2C monitoring

  /* Initialize USART service  */
  USART_Init(USART0,USART_Asynchronous,51,8,USART_ParityDisabled,
             USART_RXDIntEnable,
             USART_RXDBuffer);
             
  VT102_TextAttribute(SET_NORMAL);
  VT100_EraseInDisplay(ALL);
  VT100_CursorPosition(0,0);
  SystemLog(InitLog1);
  SystemLog(InitLog2);
  SystemLog(InitLog3);

  PINA = 0;      // set port A as OUT
  DDRA = 255;
  PORTA = 0xff;

  PINC = 0;      // set port C as OUT
  DDRC = 255;
  PORTC = 0xff;

  SystemLog(InitLog4);
  AvrInit();
  SystemLog(InitLog5);
  LCD_Init();
  LCD_Goto(0,0);
  LCD_WriteText("Initializing...");
  LCD_BarGraphInit();

  SystemLog(InitLog6);

  OSInit();  // Initialize uC/OS-II
  OSTaskCreate(StartTask, (void *)0, (void *)&StartTaskStk[TASK_STK_SIZE - 1], 3);
  OSStart();  // Start multitasking
  
  return 0;
} /* main */


